import { Component, OnInit } from '@angular/core';
import { QuestionService } from '../../services/question.service';
import { ActivatedRoute, Router } from '@angular/router';
import Swal from 'sweetalert2';

@Component({
    templateUrl: './question.component.html',
    styleUrl: './question.component.css',
})
export class QuestionComponent implements OnInit {
    constructor(
        private questionSerivce: QuestionService,
        private activatedRoute: ActivatedRoute,
        private router: Router
    ) { }
    isLoading: boolean = false;
    questions: any = [];
    actualQuestion: any;
    actualQuestionIndex: number = -1;
    showResults: boolean[] = [false, false, false, false];
    showResult: boolean = false;
    answers: string[] = [];
    score: number = 0;
    ngOnInit(): void {
        this.isLoading = true;
        this.activatedRoute.queryParams.subscribe((params) => {
            const nbQuestions = params['nbQuestions'];
            const difficulty = params['difficulty'];
            const category = params['category'];
            this.questionSerivce.getQuiz(nbQuestions, category, difficulty).subscribe({
                next: (data: any) => {
                    this.questions = data['results'];
                    this.nextQuestion();
                    this.isLoading = false;
                },
            });
        });
    }

    nextQuestion() {
        if (this.actualQuestionIndex === this.questions.length - 1) {
            Swal.fire({
                title: 'Quiz finished !',
                text: 'Want a rematch ?',
                showDenyButton: true,
                showCancelButton: false,
                confirmButtonText: 'Play again',
                confirmButtonColor: '#7e3f97',
                denyButtonColor: '#D6BAEC',
                denyButtonText: `Back home`,
            }).then((result) => {
                if (result.isConfirmed) {
                    this.actualQuestionIndex = -1;
                    this.nextQuestion();
                } else if (result.isDenied) {
                    this.router.navigate(['/start']);
                }
            });
        } else {
            this.actualQuestionIndex++;
            this.actualQuestion = this.questions[this.actualQuestionIndex];
            this.showResults = [false, false, false, false];
            this.showResult = false;
            if (this.actualQuestion.type === 'multiple') {
                this.answers = [...this.actualQuestion.incorrect_answers, this.actualQuestion.correct_answer];
                this.randomArray(this.answers);
            }
        }
    }

    decodeHtml(text: string): string {
        const textarea = document.createElement('textarea');
        textarea.innerHTML = text;
        return textarea.value;
    }

    randomArray(array: any[]): void {
        for (let i = array.length - 1; i > 0; i--) {
            const j = Math.floor(Math.random() * (i + 1));
            [array[i], array[j]] = [array[j], array[i]];
        }
    }

    handleClick(res: string, index: number) {
        if (this.showResult) return;
        console.log(res);
        if (res === this.actualQuestion.correct_answer) {
            this.score++;
        } else {
            this.answers.forEach((answer, index) => {
                if (answer === this.actualQuestion.correct_answer) {
                    this.showResults[index] = true;
                }
            });
        }
        this.showResults[index] = true;
        this.showResult = true;
    }
}
